function Utilities(ScreenService, localStorageEngine, ZHPromise, Query,
                   requestDataSourceEngine, CPLocalVars, MessageService, ZhPreferences, ZhFilterPreferences, templateManager, metaDataFactory, ZhLayoutPreferences){
    var self = this;
    displayFormat = "DD/MM/YYYY";
    var monthNames = ["Janeiro", "Fevereiro", "Março", "April", "Maio", "Junho",
        "Julho", "Agosto", "Setembro", "Outubro", "Novembro", "Dezembro"];
    this.COMMIT = "\nCOMMIT\n/";
    this.COMPILE_INVALID_OBJECTS = "\nBEGIN\n" +
        "   BEGIN\n" +
        "       EXECUTE IMMEDIATE 'ALTER SESSION SET PLSCOPE_SETTINGS = ''IDENTIFIERS:NONE''';\n" +
        "   EXCEPTION\n" +
        "       WHEN OTHERS THEN NULL;\n" +
        "   END;\n\n" +
        "   COMPILE_INVALID_OBJECTS(SYS_CONTEXT('userenv', 'current_schema'));\n" +
        "END;\n/\n";

    this.localVar = CPLocalVars;

    this.DEPLOY_ERROR_STATUS = 'D';
    this.TEST_ERROR_STATUS = 'T';
    this.PACKAGE_ERROR_STATUS = 'P';

    this.setLocalVar = function(key, value){
        localStorageEngine.setLocalVar(key, value);
    };

    this.windowResizeKanban = function(time){
        setTimeout(function(){
            window.dispatchEvent(new Event('resize'));
        }, time);
    };

    this.getLocalVar = function(key){
        return localStorageEngine.getLocalVar(key);
    };

    this.openPopup = function(url, title, w, h, params) {

        var width = window.innerWidth ? window.innerWidth : document.documentElement.clientWidth ? document.documentElement.clientWidth : screen.width;
        var height = window.innerHeight ? window.innerHeight : document.documentElement.clientHeight ? document.documentElement.clientHeight : screen.height;

        var left = ((width / 2) - (w / 2));
        var top = ((height / 2) - (h / 2));
        var newWindow = window.open(url, title, params + ', width=' + w + ', height=' + h + ', top=' + top + ', left=' + left);

        // Puts focus on the newWindow
        if (window.focus) {
            newWindow.focus();
        }
        return newWindow;
    };

    /*************************************************************
     * Gets the current system time using Moment.
     * @returns {*}
     *************************************************************/
    this.getCurrentDate = function(){
        return moment().format('DD/MM/YYYY');
    };

    /*************************************************************
     * Gets the current system date and time using Moment
     * @returns {*}
     *************************************************************/
    this.getCurrentDateAndHour = function(){
        return moment().format('DD/MM/YYYY HH:mm:ss');
    };

    /*************************************************************
     * Gets a date (doesn't matter if format is dd/mm/yyyy or
     * dd-mm-yyyy), and always returns dd-mm-yyyy.
     * @param dateTime
     * @returns {*}
     *************************************************************/
    this.formatDateTime = function(dateTime) {
        if(dateTime !== ""){
            var date = moment(dateTime, "DD-MM-YYYY HH:mm");
            return date.format("DD-MM-YYYY HH:mm");
        }
    };

    this.formatDateMonthNameAndYear = function(date){
        return ScreenService.i18n(monthNames[date.month()])+" "+date.year();
    };

    this.downLoadAttachment = function(row){
        var serviceUrl = window.serviceUrl;
        if(row.__fromIssue) {
            window.open(serviceUrl + "/downLoadAttachment?requestType=DataSet&dataset[0][ID]="+row.ID);
        }else if(row.__fromCRM) {
            window.open(row.FILENAME);
        }else if(row.__fromGEP) {
            window.open(serviceUrl + "/downLoadAttachmentGEP?requestType=DataSet&dataset[0][FILENAME]="+row.FILENAME);
        }else if(row.__fromTicket) {
            window.open(serviceUrl + "/downLoadAttachmentTicket?requestType=DataSet&dataset[0][ID]="+row.ID);
        }else {
            window.open(serviceUrl + "/downLoadAttachment?requestType=DataSet&dataset[0][NRCALLATTACHMENT]="+row.ID);
        }
    };

    this.clearValueSelect = function(field){
        field.widget.currentRow[field.name] = "";
        for (var prop in field.outData) {
            field.widget.currentRow[field.outData[prop]] = "";
        }
    };

    this.validateReadOnlyWithValidate = function(widget){
       widget.fields.forEach(function(field){
           if(field.readOnly){

           }
       });
    };

    this.deleteViewMasterDetail = function(widget) {
        return widget.dataSource.remove([widget.currentRow]).then(function(){
            ScreenService.closeSwipe();
            widget.reload();
        });
    };

    this.cancelEditMasterDetail = function(widget) {
        var primaryProperty = widget.primaryProperty ? widget.primaryProperty : 'ID';
        var primaryValue = widget.currentRow[primaryProperty];
        if (primaryValue) {
            widget.list();
            return widget.reload().then(function () {
                widget.view();
                var row = self.getRowInDataSource(widget.dataSource, primaryProperty, primaryValue);
                if(row) {
                    widget.setCurrentRow(row);
                } else {
                    self.goBackViewMasterDetail(widget);
                }
            });
        } else {
            self.goBackViewMasterDetail(widget);
        }
    };

    this.getRowInDataSource = function (dataSource, primaryProperty, primaryValue) {
        var currentRow = null;
        dataSource.data.some(function (row) {
            if (row[primaryProperty] == primaryValue) {
                currentRow = row;
                return true;
            }
        });
        return currentRow;
    };

    this.maskFormatFunction = function (value) {
        if (value) {
            var length = value.length;
            if (length === 11)
                value = value.replace(/^(\d{3})(\d{3})(\d{3})(\d{2})/, "$1.$2.$3-$4");
            else if (length === 14)
                value = value.replace(/^(\d{2})(\d{3})(\d{3})(\d{4})(\d{2})/, "$1.$2.$3/$4-$5");
        }
        return value;
    };

    this.checkLetterInString = function(string) {
        return string.toLowerCase() != string.toUpperCase();
    };

    this.validateLettersAndNumbers = function(string) {
        return /^[a-zA-Z1-9]+$/.test(string);
    };

    this.replaceSpecialCharacters = function(string, charToReplace){
        return string.replace(/[`~!@#$%^&*()_|+\-=¨?;Ç:ªº°§´`'"¹₢²³£¢¬°,.<>\{\}\[\]\\\/]/gi, charToReplace);
    };

    this.setPhoneFieldCharacters = function(fieldValue){
        return fieldValue.replace(/[^\d() -]/g, '');
    };

    this.setCepFieldCharacters = function(fieldValue){
        return fieldValue.replace(/[^\d.-]/g, '');
    };

    this.startFilter = function(widget, currentRow, isApplyOriginalFilter) {
        ZhPreferences.getDefaultWidgetPreferences(widget.id).then(function(result) {
            if (result && result.DEFAULT_FILTER) {
                ZhPreferences.getPreferences('filter', Query.build().where('WIDGET_ID').equals(widget.id)).then(function(filters) {
                    var defaultFilter = filters.filter(function(filter) {
                        return filter.ID == result.DEFAULT_FILTER;
                    })[0];
                    var filter = {
                        ID: defaultFilter.LABEL_TO_PREFERENCE || 'EDITED',
                        FILTER_ROW: JSON.parse(defaultFilter.FILTER_ROW)
                    };
                    ZhFilterPreferences.applyFilter(widget, filter).then(function() {
                        widget.appliedFilter = widget.appliedFilter || filter;
                        widget.appliedFilter.ID = result.DEFAULT_FILTER;
                        if(defaultFilter.LABEL_TO_PREFERENCE) {
                            widget._filterInfo[0].value = defaultFilter.LABEL_TO_PREFERENCE;
                        }
                    });
                });
            } else if(isApplyOriginalFilter) {
                ZhFilterPreferences.applyOriginalFilter(widget);
            } else {
                ZhFilterPreferences.openFilterWidget(widget, currentRow);
            }
        }).catch(function() {
            ZhFilterPreferences.openFilterWidget(widget, currentRow);
        });
    };

    this.isAppliedFilter = function(widget){
        return !!widget.cpAppliedFilter;
    };

    this.setFilterInit = function (widget) {
        widget.cpAppliedFilter = true;
    };

    this.applyDefaultFilterOnEnter = function (widget, isApplyOriginalFilter) {
        if (widget.inListMode()) {
            if (widget.cpAppliedFilter) {
                widget.reload();
            } else if (widget.currentRowDefaultFilter) {
                self.startFilter(widget, widget.currentRowDefaultFilter, isApplyOriginalFilter);
            } else {
                self.startFilter(widget, widget.currentRow, isApplyOriginalFilter);
            }
        }
    };

    this.fixBugMasterDetail= function(widget){
        widget.currentWidget = null;
        widget.activate();
    };

    this.initializeFieldSelectAutocomplete = function (widgetName, field) {
        var widget = templateManager.container.getWidget(widgetName);
        widget.getField(field).initializeField();
    };

    this.showMessageErrorOfBackEnd = function(response) {
        return MessageService.getMessageByCode('UNEXPECTED_ERROR').then(function(defaultMessage) {
            if (response && response.data && response.data.error) {
                var error = response.data.error;
                if (self.isJson(error)) {
                    error = JSON.parse(error);
                    var params = error.PARAMS || null;
                    var messageCode = error.CODE || 'UNEXPECTED_ERROR';
                    return MessageService.getMessageByCode(messageCode).then(function(message) {
                        return ScreenService.showMessage(message, 'ERROR', null, params);
                    }).catch(function() {
                        return ScreenService.showMessage(messageCode, 'ERROR', null, params);
                    });
                } else {
                    var message = Util.isEmptyOrBlank(error) ? defaultMessage : error.split("Exception:").pop();
                    return ScreenService.showMessage(message, 'ERROR');
                }
            } else {
                return ScreenService.showMessage(defaultMessage, 'ERROR');
            }
        });
    };

    this.showMessageBackEndTrue = function(response) {
        return self.showMessageErrorOfBackEnd(response);
    };

    this.getDeployError = function(string, type) {
        var filter = [
            {
                "name": "LOG",
                "value": string,
                "operator": "="
            },
            {
                "name": "TYPE",
                "value": type,
                "operator": "="
            }
        ];

        return requestDataSourceEngine.getDataSource('/findDeployErrors', filter).then(function (data) {
            return data.dataset.knowErrors;
        });
    };

    this.treatErrors = function(widget, log, type) {
        if (widget.fields) {
            removeAllFieldsCmGenerated(widget);
            removeAllFieldGroupsCmGenerated(widget);
        }
        this.getDeployError(log, type).then(function (response) {
            widget.currentRow = {};
            var fields = widget.fields;
            var fieldGroup = widget.fieldGroups;
            if (response.length > 0) {
                var groupCount = 0;
                response.forEach(function (error, key) {
                    if (key === 0 || error.DESCRIPTION != response[key - 1].DESCRIPTION) {
                        if (key > 0) {
                            groupCount++;
                        }
                        var groupExpression = self.getFieldGroup(error.DESCRIPTION, widget, groupCount);
                        fieldGroup.push(groupExpression);
                    }
                    var motive = self.getErrorMotiveField(widget, key, groupCount);
                    widget.currentRow["REASON_" + key] = error.REASON;
                    fields.push(motive);
                    var solution = self.getErrorSolutionField(widget, key, groupCount);
                    widget.currentRow["SOLUTION_" + key] = error.RESOLUTION;
                    fields.push(solution);
                });
                MessageService.showMessageByCode('ERR_DEPLOY_PARAM_ERR', null, {type: 'INFORMATION'});
                ScreenService.openPopup(widget);
            } else {
                self.showMessageBackEndTrue(response);
            }
        }).catch(self.showMessageBackEndTrue);
    };

    var removeAllFieldsCmGenerated = function(widget){
        widget.fields = widget.fields.filter(function(field){
            return !field.cpIsGenerated;
        });
    };

    var removeAllFieldGroupsCmGenerated = function(widget){
        widget.fieldGroups = widget.fieldGroups.filter(function(field){
            return !field.cpIsGenerated;
        });
    };
    this.getFieldGroup = function (name, widget, count) {
        return metaDataFactory.factoryField(this.buildFieldGroup(name, count), widget);
    };

    this.getErrorMotiveField = function (widget, count, fieldGroup) {
        return metaDataFactory.factoryField(this.getErrorMotive(count, fieldGroup), widget);
    };

    this.getErrorSolutionField = function (widget, count, fieldGroup) {
        return metaDataFactory.factoryField(this.getErrorSolution(count, fieldGroup), widget);
    };


    this.buildFieldGroup = function (name, count) {
        return {
            "name":  "GROUP_" + count,
            "label": name,
            "collapsible": true,
            "opened": true,
            "cpIsGenerated" : true
        };
    };

    this.getErrorMotive = function (count, fieldGroup) {
        return {
            "name": "REASON_" + count,
            "label": "Motivo",
            "isVisible": true,
            "showOnForm": true,
            "showOnView": true,
            "showOnList": true,
            "textRows": 5,
            "fieldGroup": "GROUP_" + fieldGroup,
            "readOnly": true,
            "cpIsGenerated": true,
            "template": "field/textarea.html",
            "class": 6,
            "events": []
        };
    };

    this.getErrorSolution = function (count, fieldGroup) {
        return {
            "name": "SOLUTION_" + count ,
            "label": "Resolução",
            "isVisible": true,
            "showOnForm": true,
            "showOnList": true,
            "showOnView": true,
            "textRows": 5,
            "fieldGroup": "GROUP_" + fieldGroup,
            "readOnly": true,
            "cpIsGenerated": true,
            "template": "field/textarea.html",
            "class": 6,
            "events": []
        };
    };

    this.removeCurrentRow = function(widget, messageCode){
        messageCode = messageCode || 'ALERT_GENERIC_REMOVE_MESSAGE';
        MessageService.confirmMessageByCode(messageCode, null,
            {
                type: 'QUESTION',
                success: function() {
                    widget.removeRow().then(function(){
                        ScreenService.closeSwipe();
                    });
                },
                fail: function () {}
            }
        );
    };

    this.setOutData = function(field,value){
        var vField = field.valueField;
        if (value) {
            var matchRows = field.dataSource.data.filter(function(item) {
                return item[vField] == value;
            });
            if (matchRows.length > 0) {
                Object.keys(field.outData).forEach(function(key) {
                    field.widget.currentRow[field.outData[key]] = matchRows[0][key];
                });
            }
        }
    };

    this.addNewRow = function(widget,reload){
        var deferred = ZHPromise.defer();
        if(widget.isValid()) {
            widget.saveRow().then(function (reponse) {
                if(reload){
                    widget.reload().then(function(){
                        var primaryProperty = widget.primaryProperty ? widget.primaryProperty : 'ID';
                        var primaryValue = widget.currentRow[primaryProperty];
                        widget.dataSource.data.some(function (row) {
                            if (row[primaryProperty] == primaryValue) {
                                widget.setCurrentRow(row);
                                return true;
                            }
                        });
                        widget.view();
                        widget.currentRow.changed = false;
                        deferred.resolve(reponse);
                    });
                }else{
                    widget.view();
                    widget.currentRow.changed = false;
                    deferred.resolve(reponse);
                }

            }).catch(function (reponse) {
                ScreenService.showMessage(reponse.data.error);
                deferred.reject(reponse);
            });
        } else {
            MessageService.showMessageByCode('ERR_REQUIRED_FIELDS');
            deferred.reject();
        }
        return deferred.promise;
    };

    this.goBackViewMasterDetail = function(widget) {
        widget.list();
        ScreenService.closeSwipe();
        return widget.reload();
    };

    this.getBrazilCountryCode = function(){
        return '0055';
    };

    this.applyDefaultFilter = function(widget){
        if (widget.inListMode()){
            if (self.isAppliedFilter(widget)){
                widget.reload();
            } else {
                if(widget.__filterPopup ) {
                    self.startFilter(widget, widget.__filterPopup.currentRow);
                }else if(widget.currentRowDefaultFilter){
                    self.startFilter(widget, widget.currentRowDefaultFilter);
                }else {
                    self.startFilter(widget,{});
                }
            }
        }
    };

    this.setOriginalLayout = function(widget, preferences){
        if(preferences && preferences.layout){
            return self.findPreferencesPerId(preferences.layout, "ZH_PREFERENCE_LAYOUT").then(function(layout) {
                if(layout.ID == preferences.layout){
                     templateManager.container.ORIGINAL_LAYOUT[widget.id] = JSON.parse(layout.LAYOUT_CHANGES);
                     return ZhLayoutPreferences.applyOriginalLayout(widget);
                } else {
                    return ZHPromise.when();
                }
            });
        } else {
            return ZHPromise.when();
        }
    };

    this.setOriginalView = function(widget, preferences){
        if(preferences && preferences.view){
            self.findPreferencesPerId(preferences.view, "ZH_PREFERENCE_VIEW").then(function(view) {
                if(view.ID == preferences.view){
                    preferences.filter = view.FILTER_ID;
                    preferences.layout = view.LAYOUT_ID;
                    self.setOriginalLayout(widget, preferences).then(function() {
                        self.setOriginalFilter(widget, preferences);
                    });
                }
            });
        }
    };


    this.findPreferencesPerId = function(id, type){
         var filters = [ {
            name: "ID",
            value: id,
            operator: "="
        }, {
            name: "TYPE",
            value: type,
            operator: "="
        } ];
        return requestDataSourceEngine.getDataSource('/findPreferencesPerId', filters).then(function(preferences){
            return preferences.dataset.findPreferencesPerId[0];
        });
    };

    this.setOriginalFilter = function(widget, preferences) {
        if(preferences && preferences.filter){
            self.findPreferencesPerId(preferences.filter, "ZH_PREFERENCE_FILTER").then(function(filter) {
                if(filter.ID == preferences.filter) {
                    ZhFilterPreferences.setOriginalFilter(widget.id, JSON.parse(filter.FILTER_ROW));
                    self.applyDefaultFilterOnEnter(widget, true);
                }
            });
        } else {
             ZhFilterPreferences.setOriginalFilter(widget.id, widget.currentRow);
             self.applyDefaultFilterOnEnter(widget);
        }
    };

    this.applyFilter = function(widget, currentRow){
        ZhFilterPreferences.setOriginalFilter(widget, currentRow);
    };

    this.isRequiredField = function(field){
        return field.hasOwnProperty('validations') && field.validations.hasOwnProperty('required');
    };

    this.buildDefaultFilterByFilterProperties = function(widget){
        var defaultFilter = {};
        widget.fields.forEach(function(field){
            if(field.filterProperties && field.filterProperties.defaultValue){
                if(field.filterProperties.name){
                    defaultFilter[field.filterProperties.name] = field.filterProperties.defaultValue;
                } else {
                    defaultFilter[field.name] = field.filterProperties.defaultValue;
                }
            }
        });
        return defaultFilter;
    };

    this.setDescriptionOrder = function(widget) {
        widget.descriptionOrder = widget.parent.descriptionOrder;
    };

    this.doSyncCP = function(funcions) {
        return funcions.reduce(function(lastPromise, nextFunction) {
            return lastPromise.then(function() {
                return nextFunction();
            });
        }, ZHPromise.when());
    };

    this.getCurrentDateWithTime = function () {
        return moment().format('DD/MM/YYYY HH:mm:ss');
    };

    this.getCurrentDateInMilliseconds = function(){
        return new Date().getTime();
    };

    this.getCurrentFormattedDateForAttachment = function(){
        return moment().format('YYYY-MM-DDTHH:MM:SSZ');
    };

    this.createLogFile = function(logFileName, logFileContents, idField, value) {
        var params = buildLog(logFileName, idField, value);
        params = prepareFileSizeAndContent(params, logFileContents);
        return requestDataSourceEngine.setDataSource('/saveLog', params);
    };

    var buildLog = function(logFileName, idField, value){
        var fileObject = [ {
            __createdLocal: true,
            '__is_new': true,
            changed: true,
            DOCUMENT : [{
                'name': logFileName,
                'b64File': '',
                'lastModified': self.getCurrentDateInMilliseconds(),
                'lastModifiedDate': self.getCurrentFormattedDateForAttachment(),
                'size': 0,
                'type': 'text/plain',
                'webkitRelativePath': ''
            }]
        } ];
        fileObject[0][idField] = value;
        return fileObject;
    };

    var prepareFileSizeAndContent = function(params, logFileContents){
        var fileType = 'data:text/plain;base64,';
        logFileContents = self.transliterateString(logFileContents);
        params[0].DOCUMENT[0].b64File = fileType + window.btoa(logFileContents.replace(/[\u00A0-\u2666]/g, ''));
        params[0].DOCUMENT[0].size = self.byteLength(logFileContents);
        return params;
    };

    this.byteLength = function(str) {
        var s = str.length;
        for (var i=str.length-1; i>=0; i--) {
            var code = str.charCodeAt(i);
            if (code > 0x7f && code <= 0x7ff) s++;
            else if (code > 0x7ff && code <= 0xffff) s+=2;
            if (code >= 0xDC00 && code <= 0xDFFF) i--;
        }
        return s;
    };

    this.getLogFileContents = function(logFileName) {
        var filter = [{
            "name": "FILE_NAME",
            "value": logFileName,
            "operator": "="
        }];
        return requestDataSourceEngine.getDataSource('/getFullLog', filter).then(function(data) {
            return data.dataset.logFileContents.length > 0 ? data.dataset.logFileContents[0] : false;
        });
    };

    this.sendLogFileContents = function(logFileName, widget) {
        var row = widget.currentRow;
        widget.currentRow.LOG_FILE_NAME = logFileName;
        return requestDataSourceEngine.setDataSource('/sendLogByEmail', [row]);
    };

    this.transliterateString = function(text) {
        var transliterationTable = {
            "á" : "a",  "Á" : "A",  "à" : "a",  "À" : "A",  "a" : "a",
            "A" : "A",  "â" : "a",  "Â" : "A",  "å" : "a",  "Å" : "A",
            "ã" : "a",  "Ã" : "A",  "ä" : "ae", "Ä" : "AE", "æ" : "ae",
            "Æ" : "AE", "c" : "c",  "C" : "C",  "ç" : "c",  "Ç" : "C",
            "d" : "d",  "D" : "D",  "Ð" : "Dh", "ð" : "dh", "é" : "e",
            "É" : "E",  "è" : "e",  "È" : "E",  "e" : "e",  "E" : "E",
            "ê" : "e",  "Ê" : "E",  "ë" : "e",  "Ë" : "E",  "ƒ" : "F",
            "g" : "g",  "G" : "G",  "h" : "h",  "H" : "H",  "í" : "i",
            "Í" : "I",  "ì" : "i",  "Ì" : "I",  "î" : "i",  "Î" : "I",
            "ï" : "i",  "Ï" : "I",  "i" : "i",  "I" : "I",  "j" : "j",
            "J" : "J",  "k" : "k",  "K" : "K",  "l" : "l",  "L" : "L",
            "n" : "n",  "N" : "N",  "ñ" : "n",  "Ñ" : "N",  "ó" : "o",
            "Ó" : "O",  "ò" : "o",  "Ò" : "O",  "ô" : "o",  "Ô" : "O",
            "o" : "o",  "O" : "O",  "õ" : "o",  "Õ" : "O",  "ø" : "oe",
            "Ø" : "OE", "ö" : "oe", "Ö" : "OE", "r" : "r",  "R" : "R",
            "s" : "s",  "S" : "S",  "š" : "s",  "Š" : "S",  "ß" : "SS",
            "t" : "t",  "T" : "T",  "ú" : "u",  "Ú" : "U",  "ù" : "u",
            "Ù" : "U",  "u" : "u",  "U" : "U",  "û" : "u",  "Û" : "U",
            "ü" : "ue", "Ü" : "UE", "w" : "w",  "W" : "W",  "ý" : "y",
            "Ý" : "Y",  "y" : "y",  "Y" : "Y",  "ÿ" : "y",  "Ÿ" : "Y",
            "z" : "z",  "Z" : "Z",  "ž" : "z",  "Ž" : "Z",  "þ" : "th",
            "Þ" : "Th", "µ" : "u"
        };
        var newText = "";
        if(text){
            for(var i = 0; i < text.length; i++){
                newText += transliterationTable[text[i]] || text[i];
            }
        }
        return newText;
    };

    this.getValueInSelectField = function(field){
        return field.widget.currentRow[field.outData[field.valueField]];
    };

    this.getCurrentRowInFieldSelect = function(field){
        var value = self.getValueInSelectField(field);
        var row = null;
        if(value){
            row = self.findInDataSource(field,[value],field.valueField)[0];
        }
        return row;
    };


    this.onlyNumbers = function(string)
    {
        var numsStr = string.replace(/[^0-9]/g,'');
        return numsStr;
    };

    this.findInDataSource = function(field,values,index){
        return field.dataSource.data.filter(function(row){
            var found = false;
            values.forEach(function(value){
                if(row[index] == value){
                    found = true;
                }
            });
            return found;
        });
    };

    this.isJson = function(str) {
        var result = null;

        try {
            result = JSON.parse(str);
        } catch (e) { }

        return result;
    };
    this.windowResize = function(time){
        setTimeout(function(){
            window.dispatchEvent(new Event('resize'));
        }, time);
    };

    /**
     * Carrega os registros da widget passada quando ela estiver em modo list.
     *
     * @param {object} widget
     */
    this.carregarWidgetOnEnterTab = function(widget) {
        if (widget.mode == "list") widget.reload();
    };

    this.setProductModulesData = function (productModulesWidget) {
        productModulesWidget.dataSource.data = productModulesWidget.productModulesData;
        productModulesWidget.currentRow = productModulesWidget.productModulesData[0] || {};
        productModulesWidget.currentRow._opened = true;
        templateManager.updateTemplate();
    };

    this.isDisabledDeploy = function(row) {
        var login = localStorageEngine.getLocalVar('customerPortalLogin');
        return (
            !login.isTeknisaUser &&
            (
                (row.IDORGSAAS === "S" && row.IDPRODUTOSAAS !== "N" ) ||
                (row.IDORGSAAS !== "S" &&
                    (row.IDPRODUTOSAAS === "S" ||
                        (row.IDPRODUTOSAAS === "O" && row.IDPRODUTOORGSAAS === "S")
                    )
                )
            )
        );
    };

    this.checkPermissionToDeploy = function() {
        var permissionToDeployArray = permissionToDeploy.split(',');
        var cdCliente = localStorageEngine.getLocalVar('CDCLIENTE');
        return permissionToDeployArray.includes(cdCliente) || permissionToDeploy == 'ALL';
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('Utilities', Utilities);
});
